#if 0
''' '
#endif

#ifdef __cplusplus
template <typename T>
using Pin = T;
template <typename T>
using Box = T*;
#endif

#if 0
' '''
#endif


from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  cdef struct PinTest:
    int32_t *pinned_box;
    int32_t *pinned_ref;

  void root(int32_t *s, PinTest p);



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  #With doc attr, each attr contribute to one line of document
  #like this one with a new line character at its end
  #and this one as well. So they are in the same paragraph
  #
  #Line ends with one new line should not break
  #
  #Line ends with two spaces and a new line
  #should break to next line
  #
  #Line ends with two new lines
  #
  #Should break to next paragraph
  void root();



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  ctypedef struct TypedLength_f32__UnknownUnit:
    float _0;

  ctypedef struct TypedLength_f32__LayoutUnit:
    float _0;

  ctypedef TypedLength_f32__UnknownUnit Length_f32;

  ctypedef TypedLength_f32__LayoutUnit LayoutLength;

  ctypedef struct TypedSideOffsets2D_f32__UnknownUnit:
    float top;
    float right;
    float bottom;
    float left;

  ctypedef struct TypedSideOffsets2D_f32__LayoutUnit:
    float top;
    float right;
    float bottom;
    float left;

  ctypedef TypedSideOffsets2D_f32__UnknownUnit SideOffsets2D_f32;

  ctypedef TypedSideOffsets2D_f32__LayoutUnit LayoutSideOffsets2D;

  ctypedef struct TypedSize2D_f32__UnknownUnit:
    float width;
    float height;

  ctypedef struct TypedSize2D_f32__LayoutUnit:
    float width;
    float height;

  ctypedef TypedSize2D_f32__UnknownUnit Size2D_f32;

  ctypedef TypedSize2D_f32__LayoutUnit LayoutSize2D;

  ctypedef struct TypedPoint2D_f32__UnknownUnit:
    float x;
    float y;

  ctypedef struct TypedPoint2D_f32__LayoutUnit:
    float x;
    float y;

  ctypedef TypedPoint2D_f32__UnknownUnit Point2D_f32;

  ctypedef TypedPoint2D_f32__LayoutUnit LayoutPoint2D;

  ctypedef struct TypedRect_f32__UnknownUnit:
    TypedPoint2D_f32__UnknownUnit origin;
    TypedSize2D_f32__UnknownUnit size;

  ctypedef struct TypedRect_f32__LayoutUnit:
    TypedPoint2D_f32__LayoutUnit origin;
    TypedSize2D_f32__LayoutUnit size;

  ctypedef TypedRect_f32__UnknownUnit Rect_f32;

  ctypedef TypedRect_f32__LayoutUnit LayoutRect;

  ctypedef struct TypedTransform2D_f32__UnknownUnit__LayoutUnit:
    float m11;
    float m12;
    float m21;
    float m22;
    float m31;
    float m32;

  ctypedef struct TypedTransform2D_f32__LayoutUnit__UnknownUnit:
    float m11;
    float m12;
    float m21;
    float m22;
    float m31;
    float m32;

  void root(TypedLength_f32__UnknownUnit length_a,
            TypedLength_f32__LayoutUnit length_b,
            Length_f32 length_c,
            LayoutLength length_d,
            TypedSideOffsets2D_f32__UnknownUnit side_offsets_a,
            TypedSideOffsets2D_f32__LayoutUnit side_offsets_b,
            SideOffsets2D_f32 side_offsets_c,
            LayoutSideOffsets2D side_offsets_d,
            TypedSize2D_f32__UnknownUnit size_a,
            TypedSize2D_f32__LayoutUnit size_b,
            Size2D_f32 size_c,
            LayoutSize2D size_d,
            TypedPoint2D_f32__UnknownUnit point_a,
            TypedPoint2D_f32__LayoutUnit point_b,
            Point2D_f32 point_c,
            LayoutPoint2D point_d,
            TypedRect_f32__UnknownUnit rect_a,
            TypedRect_f32__LayoutUnit rect_b,
            Rect_f32 rect_c,
            LayoutRect rect_d,
            TypedTransform2D_f32__UnknownUnit__LayoutUnit transform_a,
            TypedTransform2D_f32__LayoutUnit__UnknownUnit transform_b);



# Copyright (C) 2020-2024 Cisco Systems, Inc. and/or its affiliates. All rights reserved.

"""
Run clamscan tests.
"""

import sys

sys.path.append('../unit_tests')
import testcase


class TC(testcase.TestCase):
    @classmethod
    def setUpClass(cls):
        super(TC, cls).setUpClass()

        (TC.path_tmp / 'monitor-example-com.pdb').write_text('H:example.com\n')

    @classmethod
    def tearDownClass(cls):
        super(TC, cls).tearDownClass()

    def setUp(self):
        super(TC, self).setUp()

    def tearDown(self):
        super(TC, self).tearDown()
        self.verify_valgrind_log()

    def test_not_enabled(self):
        self.step_name('Test that clamscan will load the phishing sigs w/out issue')

        testpaths = list(TC.path_source.glob('unit_tests/input/other_scanfiles/phish-test-*'))

        testfiles = ' '.join([str(testpath) for testpath in testpaths])
        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args,
            clamscan=TC.clamscan,
            path_db=TC.path_tmp / 'monitor-example-com.pdb',
            testfiles=testfiles,
        )
        output = self.execute_command(command)

        assert output.ec == 0  # virus NOT found

        expected_results = [
            'Scanned files: 3',
            'Infected files: 0',
        ]
        self.verify_output(output.out, expected=expected_results)

    def test_ssl_and_cloak(self):
        self.step_name('Test clamscan --alert-phishing-ssl --alert-phishing-cloak')

        testpaths = list(TC.path_source.glob('unit_tests/input/other_scanfiles/phish-test-*'))

        testfiles = ' '.join([str(testpath) for testpath in testpaths])
        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} --alert-phishing-ssl --alert-phishing-cloak {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args,
            clamscan=TC.clamscan,
            path_db=TC.path_tmp / 'monitor-example-com.pdb',
            testfiles=testfiles,
        )
        output = self.execute_command(command)

        assert output.ec == 1  # virus found

        expected_results = [
            'phish-test-ssl: Heuristics.Phishing.Email.SSL-Spoof FOUND',
            'phish-test-cloak: Heuristics.Phishing.Email.Cloaked.Null FOUND',
            'Scanned files: 3',
            'Infected files: 2', # there's a clean one
        ]
        self.verify_output(output.out, expected=expected_results)



# Copyright (C) 2020-2024 Cisco Systems, Inc. and/or its affiliates. All rights reserved.

"""
Run clamscan tests.
"""

import shutil
import unittest
import sys
from zipfile import ZIP_DEFLATED, ZipFile

sys.path.append('../unit_tests')
import testcase


class TC(testcase.TestCase):
    @classmethod
    def setUpClass(cls):
        super(TC, cls).setUpClass()

        TC.testpaths = list(TC.path_build.glob('unit_tests/input/clamav_hdb_scanfiles/clam*')) # A list of Path()'s of each of our generated test files

    @classmethod
    def tearDownClass(cls):
        super(TC, cls).tearDownClass()

    def setUp(self):
        super(TC, self).setUp()

    def tearDown(self):
        super(TC, self).tearDown()
        self.verify_valgrind_log()

    def test_00_version(self):
        self.step_name('clamscan version test')

        command = '{valgrind} {valgrind_args} {clamscan} -V'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan
        )
        output = self.execute_command(command)

        assert output.ec == 0  # success

        expected_results = [
            'ClamAV {}'.format(TC.version),
        ]
        self.verify_output(output.out, expected=expected_results)

    def test_weak_indicator_icon(self):
        self.step_name('Test icon (.ldb + .idb) weak indicator matching signatures')

        (TC.path_tmp / 'icon.idb').write_text(
            "EA0X-32x32x8:ea0x-grp1:ea0x-grp2:2046f030a42a07153f4120a0031600007000005e1617ef0000d21100cb090674150f880313970b0e7716116d01136216022500002f0a173700081a004a0e\n"
            "IScab-16x16x8:iscab-grp1:iscab-grp2:107b3000168306015c20a0105b07060be0a0b11c050bea0706cb0a0bbb060b6f00017c06018301068109086b03046705081b000a270a002a000039002b17\n"
        )
        (TC.path_tmp / 'icon.ldb').write_text(
            "ClamAV-Test-Icon-EA0X;Engine:52-1000,Target:1,IconGroup1:ea0x-grp1,IconGroup2:*;(0);0:4d5a\n"
            "ClamAV-Test-Icon-IScab;Engine:52-1000,Target:1,IconGroup2:iscab-grp2;(0);0:4d5a\n"
        )

        testfiles = ' '.join([str(testpath) for testpath in TC.testpaths])
        command = '{valgrind} {valgrind_args} {clamscan} -d {path_ldb} -d {path_idb} {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args,
            clamscan=TC.clamscan,
            path_ldb=TC.path_tmp / 'icon.ldb',
            path_idb=TC.path_tmp / 'icon.idb',
            testfiles=testfiles,
        )
        output = self.execute_command(command)

        assert output.ec == 1  # virus found

        # Use check_fpu_endian to determine expected results
        command = '{}'.format(TC.check_fpu_endian)
        fpu_endian_output = self.execute_command(command)

        expected_results = [
            'clam_IScab_ext.exe: ClamAV-Test-Icon-IScab.UNOFFICIAL FOUND',
            'clam_IScab_int.exe: ClamAV-Test-Icon-IScab.UNOFFICIAL FOUND',
        ]
        if fpu_endian_output.ec == 3:
            expected_num_infected = 3
        else:
            expected_results.append('clam.ea06.exe: ClamAV-Test-Icon-EA0X.UNOFFICIAL FOUND')
            expected_num_infected = 4
        expected_results.append('Infected files: {}'.format(expected_num_infected))
        self.verify_output(output.out, expected=expected_results)

    def test_pe_cert_trust(self):
        self.step_name('Test that clam can trust an EXE based on an authenticode certificate check.')

        test_path = TC.path_source / 'unit_tests' / 'input' / 'pe_allmatch'
        test_exe = test_path / 'test.exe'

        command = '{valgrind} {valgrind_args} {clamscan} \
             -d {alerting_dbs} \
             -d {weak_dbs} \
             -d {broken_dbs} \
             -d {trust_dbs} \
             --allmatch --bytecode-unsigned {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan,
            alerting_dbs=test_path / 'alert-sigs',
            weak_dbs=test_path / 'weak-sigs',
            broken_dbs=test_path / 'broken-sigs',
            trust_dbs=test_path / 'trust-sigs',
            testfiles=test_exe,
        )
        output = self.execute_command(command)

        assert output.ec == 0

        expected_results = ['OK']

        # The alert sig files are all given the signature name, so we can verify that the correct sigs were found.
        # We need only to trim off the extension and say "FOUND" for the alerting sigs.
        # Note: Some of these have ".UNOFFICIAL" in the name because not all of them have that ".UNOFFICIAL" suffix when reported.
        #       I think this is a minor bug. So if we change that, we'll need to update this test.
        unexpected_results = ['{sig} FOUND'.format(sig=f.stem) for f in (test_path / 'alert-sigs').iterdir()]

        self.verify_output(output.out, expected=expected_results, unexpected=unexpected_results)

    def test_pe_cert_block(self):
        self.step_name('Test that clam will disregard a certificate trust signature if a block certificate rule is used.')

        # The sig set and test.exe for test set was written by one of our threat researchers to test the allmatch option.
        # Overall, it's much more thorough than previous tests, but some of the tests are duplicates of the previous tests.

        # TODO: The section signatures are not working as written, hence the "broken_dbs" directory.
        #       There is a known issue with relative offset signatures when using the Boyer-Moore matcher. The sigs work if using the Aho-Corasick matcher.
        #       When we fix section signatures, we can move them to the alerting sigs directory and update this test.

        test_path = TC.path_source / 'unit_tests' / 'input' / 'pe_allmatch'
        test_exe = test_path / 'test.exe'

        command = '{valgrind} {valgrind_args} {clamscan} \
             -d {alerting_dbs} \
             -d {weak_dbs} \
             -d {broken_dbs} \
             -d {block_cert_dbs} \
             --allmatch --bytecode-unsigned {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan,
            alerting_dbs=test_path / 'alert-sigs',
            block_cert_dbs=test_path / 'block-cert-sigs',
            weak_dbs=test_path / 'weak-sigs',
            broken_dbs=test_path / 'broken-sigs',
            trust_dbs=test_path / 'trust-sigs',
            testfiles=test_exe,
        )
        output = self.execute_command(command)

        assert output.ec == 1

        # The alert sig files are all given the signature name, so we can verify that the correct sigs were found.
        # We need only to trim off the extension and say "FOUND" for the alerting sigs.
        # Note: Some of these have ".UNOFFICIAL" in the name because not all of them have that ".UNOFFICIAL" suffix when reported.
        #       I think this is a minor bug. So if we change that, we'll need to update this test.
        expected_results = ['{sig} FOUND'.format(sig=f.stem) for f in (test_path / 'alert-sigs').iterdir()]
        expected_results += ['{sig} FOUND'.format(sig=f.stem) for f in (test_path / 'block-cert-sigs').iterdir()]

        # The broken sig files are all given the signature name, so we can verify that the correct sigs were found.
        # TODO: When we fix section signatures, we can move them to the alerting sigs directory and get rid of this line.
        unexpected_results = ['{sig} FOUND'.format(sig=f.stem) for f in (test_path / 'broken-sigs').iterdir()]

        self.verify_output(output.out, expected=expected_results, unexpected=unexpected_results)

    def test_pe_cert_trust_archive(self):
        self.step_name('Test that clam\'s trust of an EXE based on a cert check doesn\'t trust a whole archive.')

        test_path = TC.path_source / 'unit_tests' / 'input' / 'pe_allmatch'

        # This file we'll trust.
        test_exe = test_path / 'test.exe'

        # This file we'll match on for an alert
        clam_exe = TC.path_build / 'unit_tests' / 'input' / 'clamav_hdb_scanfiles' / 'clam.exe'

        # Build a ZIP that first has file that we trust, followed by a file we would alert on.
        trusted_plus_mal_zip = TC.path_tmp / 'trust_plus_mal.zip'
        with ZipFile(str(trusted_plus_mal_zip), 'w', ZIP_DEFLATED) as zf:
            zf.writestr('test.exe', test_exe.read_bytes())
            zf.writestr('clam.exe', clam_exe.read_bytes())

        # Build another ZIP, but with files added in reverse order, for good measure.
        trusted_plus_mal_zip_2 = TC.path_tmp / 'trust_plus_mal2.zip'
        with ZipFile(str(trusted_plus_mal_zip_2), 'w', ZIP_DEFLATED) as zf:
            zf.writestr('clam.exe', clam_exe.read_bytes())
            zf.writestr('test.exe', test_exe.read_bytes())

        command = '{valgrind} {valgrind_args} {clamscan} \
             -d {alerting_dbs} \
             -d {weak_dbs} \
             -d {broken_dbs} \
             -d {trust_dbs} \
             -d {clamav_hdb} \
             --allmatch --bytecode-unsigned {testfile1} {testfile2}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan,
            alerting_dbs=test_path / 'alert-sigs',
            weak_dbs=test_path / 'weak-sigs',
            broken_dbs=test_path / 'broken-sigs',
            trust_dbs=test_path / 'trust-sigs',
            clamav_hdb=TC.path_source / 'unit_tests' / 'input' / 'clamav.hdb',
            testfile1=trusted_plus_mal_zip,
            testfile2=trusted_plus_mal_zip_2,
        )
        output = self.execute_command(command)

        assert output.ec == 1

        expected_results = [
            'trust_plus_mal.zip: ClamAV-Test-File.UNOFFICIAL FOUND',
            'trust_plus_mal2.zip: ClamAV-Test-File.UNOFFICIAL FOUND',
        ]
        unexpected_results = ['OK']

        self.verify_output(output.out, expected=expected_results, unexpected=unexpected_results)

    def test_iso_missing_joliet(self):
        self.step_name('Test that we correctly extract files from an ISO even if the joliet file path is empty.')

        test_path = TC.path_source / 'unit_tests' / 'input' / 'other_scanfiles'
        sig_path = TC.path_source / 'unit_tests' / 'input' / 'other_sigs' / 'logo.hsb'

        command = '{valgrind} {valgrind_args} {clamscan} \
             -d {sig_path} \
             --allmatch {testfile1} {testfile2}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan,
            sig_path=sig_path,
            testfile1=test_path / 'iso_normal.logo.iso',
            testfile2=test_path / 'iso_no_joliet.logo.iso',
        )
        output = self.execute_command(command)

        assert output.ec == 1

        expected_results = [
            'iso_normal.logo.iso: logo.png.UNOFFICIAL FOUND',
            'iso_no_joliet.logo.iso: logo.png.UNOFFICIAL FOUND',
        ]
        unexpected_results = ['OK']

        self.verify_output(output.out, expected=expected_results, unexpected=unexpected_results)

    def test_onenote_disabled(self):
        self.step_name('Test that clamscan --scan-onenote=no disables onenote support')

        testpaths = [
            TC.path_build / "unit_tests" / "input" / "clamav_hdb_scanfiles" / "clam.exe.2007.one",
            TC.path_build / "unit_tests" / "input" / "clamav_hdb_scanfiles" / "clam.exe.2010.one",
            TC.path_build / "unit_tests" / "input" / "clamav_hdb_scanfiles" / "clam.exe.webapp-export.one",
        ]

        testfiles = ' '.join([str(testpath) for testpath in testpaths])

        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args,
            clamscan=TC.clamscan,
            path_db=TC.path_build / 'unit_tests' / 'input' / 'clamav.hdb',
            testfiles=testfiles,
        )
        output = self.execute_command(command)

        assert output.ec == 1  # virus found

        expected_results = ['{}: ClamAV-Test-File.UNOFFICIAL FOUND'.format(testpath.name) for testpath in testpaths]
        expected_results.append('Scanned files: {}'.format(len(testpaths)))
        expected_results.append('Infected files: {}'.format(len(testpaths)))
        self.verify_output(output.out, expected=expected_results)

        # Try again with onenote support disabled.

        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} --scan-onenote=no {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args,
            clamscan=TC.clamscan,
            path_db=TC.path_build / 'unit_tests' / 'input' / 'clamav.hdb',
            testfiles=testfiles,
        )
        output = self.execute_command(command)

        assert output.ec == 0  # virus found

        expected_results = ['{}: OK'.format(testpath.name) for testpath in testpaths]
        expected_results.append('Scanned files: 3')
        expected_results.append('Infected files: 0')
        self.verify_output(output.out, expected=expected_results)



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  const int64_t CONSTANT_I64 # = 216

  const float CONSTANT_FLOAT32 # = 312.292

  const uint32_t DELIMITER # = ':'

  const uint32_t LEFTCURLY # = '{'

  cdef struct Foo:
    int32_t x;
  const int64_t Foo_CONSTANT_I64_BODY # = 216

  const Foo SomeFoo # = <Foo>{ 99 }



#!/bin/env python3

# Copyright (C) 2022-2024 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
#
# Authors: Andrew Williams
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

'''
Build a `test.exe` program with distinctive features that may be easily detected by clamscan.
This program is also signed with an authenticode certificate to test authenticode certificate trust features.
'''

import errno
import os
import argparse
import random

parser = argparse.ArgumentParser()
parser.add_argument("os", choices=['windows', 'linux', 'win32', 'win64'], nargs='?', default='windows')
parser.add_argument("--no-sign", action='store_true', default=False)
parser.add_argument("--no-strip", action='store_true', default=False)
parser.add_argument("--no-cleanup", action='store_true', default=False)
args = parser.parse_args()

do_cleanup = not args.no_cleanup

if args.os == 'windows' or args.os == 'win32':
    prefix = 'i686-w64-mingw32-'
    do_res = True
    do_sign = not args.no_sign
    do_strip = not args.no_strip
    # __USE_MINGW_ANSI_STDIO lets %xd be used correctly when doing printf
    cc_flags = '-D__USE_MINGW_ANSI_STDIO=1'
    ext = ".exe"

elif args.os == 'win64':
    prefix = 'x86_64-w64-mingw32-'
    do_res = True
    do_sign = not args.no_sign
    do_strip = not args.no_strip
    cc_flags = '-D__USE_MINGW_ANSI_STDIO=1'
    ext = ".exe"

else: # linux
    prefix = ''
    do_res = False
    do_sign = False
    do_strip = False
    cc_flags = ''
    ext = ''

windres = '%swindres' % (prefix)
gcc = '%sgcc' % (prefix)
ld = '%sld' % (prefix)
strip = '%sstrip' % (prefix)

indicators = [
  # Indicators for the outer binary
  ('22df_62ba_15c8_f482', '1a0d_301b_228d_56c8'),

  # Indicators for up to 24 inner binaries
  ('ffa8_3994_1788_e8b6', 'e4d7_7b43_3155_040a'),
  ('3a28_3628_9dd8_9c32', '3da0_c7bc_1948_39bd'),
  ('d251_d598_8e5c_6f9d', 'ee01_7f34_e9c9_1ec1'),
  ('6761_4a90_365b_b4c0', 'b8b0_d765_3df3_e550'),
  ('7939_5658_544a_f991', '768b_f5e5_f5e4_1a3b'),
  ('d765_7e8b_90b9_4a2f', '6cb8_1966_8bc3_9874'),
  ('cef1_e295_6e46_f429', '0ef3_77f1_e811_2753'),
  ('8f8a_f7f2_cdd1_9d0b', '99aa_e67d_afb6_3735'),
  ('d183_4b76_bde9_f5fb', 'f5e2_acf2_ff78_88f7'),
  ('3bce_e84b_be85_77bc', 'b155_ac2c_22f6_cb9e'),
  ('6efd_bf13_3679_9d30', '00de_ce35_6606_114c'),
  ('6ef7_8f1c_19f8_9746', '38d9_f9c7_45de_9931'),
  ('54b5_a0a4_b852_d3ba', 'cc78_0d79_8e9d_6f8a'),
  ('7468_e227_c130_b48d', '0c57_4d6a_e113_e03a'),
  ('8272_e8e2_a133_2971', 'e113_d12a_266c_3253'),
  ('b552_7c09_8b0f_01c7', '01c3_a69a_899f_0764'),
  ('04a3_b125_45ed_2a8b', '58f8_6bf7_b1bb_c7b6'),
  ('5749_2c8c_9df1_4c7e', 'c0a0_3f27_99c5_51f4'),
  ('89df_0268_6c60_29fb', 'db58_4b5e_f2df_9b65'),
  ('c2cd_3a28_5180_dc7a', '2c70_5359_0a46_06fa'),
  ('6764_76f5_0e92_faa6', 'd07a_b1af_e36f_8894'),
  ('4e45_18c0_40da_e74d', '6a1d_d382_068a_7e71'),
  ('01c4_23f0_0e32_fad1', 'a2f0_7a46_5cf1_2e0d'),
  ('1cec_e7c0_daac_517f', '448a_2336_facf_e5e7'),
]

if len(indicators)-1 > 32:
    raise Exception("More embedded binaries than extract.h can handle")

try:
    os.mkdir('build')
except OSError as exc:
    if exc.errno != errno.EEXIST:
        raise
    pass

def run_cmd(cmd):
    print(cmd)
    if os.system(cmd):
        raise Exception("Command Failed: %s" % cmd)

def gen_ca_cert():
    run_cmd('openssl genrsa -out build/ca.key 4096')

    # TODO Explore making this cert have attributes that look more like
    # a real CA cert (ex: restrict its uses)
    subj = "/C=US/ST=Maryland/L=Fulton/O=Cisco Talos/OU=ClamAV Test CA %016x/emailAddress=rfc2606@example.net" % (random.randint(1,0xFFFFFFFFFFFFFFFF))
    cmd = 'openssl req -new -x509 -days 73000 -key build/ca.key -out build/ca.crt -subj "%s"' % (subj)
    run_cmd(cmd)

# https://blog.didierstevens.com/2008/12/30/howto-make-your-own-cert-with-openssl/
def gen_cs_cert(name, ext):
    key_name = 'build/%s%s.key' % (name, ext)
    csr_name = 'build/%s%s.csr' % (name, ext)
    crt_name = 'build/%s%s.crt' % (name, ext)

    run_cmd('openssl genrsa -out %s 4096' % (key_name))

    # TODO Explore making this cert have attributes that look more like
    # a real CS cert (ex: restrict its uses)
    subj = "/C=US/ST=Maryland/L=Fulton/O=Cisco Talos/OU=ClamAV Test %016x/emailAddress=rfc2606@example.net" % (random.randint(1,0xFFFFFFFFFFFFFFFF))
    cmd = 'openssl req -new -key %s -out %s -subj "%s"' % (key_name, csr_name, subj)
    run_cmd(cmd)

    cmd = 'openssl x509 -req -days 73000 -in %s -CA build/ca.crt -CAkey build/ca.key -out %s -set_serial %012d -extfile ./cs.extfile.cfg' % (csr_name, crt_name, random.randint(100000000000,999999999999))
    run_cmd(cmd)

    return (key_name, crt_name)

# https://blog.didierstevens.com/2018/09/24/quickpost-signing-windows-executables-on-kali/
def sign_exe(name, ext, cert_info):
    key_name = cert_info[0]
    crt_name = cert_info[1]
    orig_path = "build/%s%s" % (name, ext)
    signed_path = "build/%s-signed%s" % (name, ext)
    cmd = 'osslsigncode sign -certs %s -key %s -ts http://sha256timestamp.ws.symantec.com/sha256/timestamp -in %s -out %s' % (crt_name, key_name, orig_path, signed_path)
    run_cmd(cmd)
    os.unlink(orig_path)
    os.rename(signed_path, orig_path)

def build_exe(name, ext, index, cc_flags, do_res, do_strip, sources=['./test.c']):

    if do_res:
        # TODO Generate a new icon and new version information per exe.  The
        # icon can be generated with:
        #
        # head -c 245760 /dev/urandom | convert -depth 8 -size 320x256 RGB:- test.png
        # convert -background transparent test.png -define icon:auto-resize=16,32,48,64,256 test.ico
        #
        # NOTE ^^^ 245760 is 320x256x3
        res_path = 'build/%s%s.res' % (name, ext)
        cmd = '%s ./test.rc -O coff -o %s' % (windres, res_path)
        run_cmd(cmd)
        sources.append(res_path)

    cmd = '%s -Os %s %s -DINDICATOR1=\\"%s\\" -DINDICATOR2=\\"%s\\" -DINDEX=%d -o build/%s%s' % (gcc, cc_flags, ' '.join([x for x in sources]), indicators[index][0], indicators[index][1], index, name, ext)
    run_cmd(cmd)

    if do_strip:
        cmd = '%s --strip-all build/%s%s' %(strip, name, ext)
        run_cmd(cmd)

def package_exe(name, ext):
    # The name of the export symbols depends on the path, so we have to be in
    # the same directory as the exe we are packaging for the symbol name the
    # code expects to be generated.
    os.chdir('build')
    cmd = '%s -r -b binary %s%s -o %s%s.o' %(ld, name, ext, name, ext)
    run_cmd(cmd)
    os.chdir('..')

# Generate a new CA cert for code-signing
if do_sign:
    gen_ca_cert()

outer_exe_sources = ['./test.c']
# Build the inner exes. They MUST be named 'exe#', where # is the index. They
# can't have a file extension or else it breaks linking.
for i in range(1, len(indicators)):
    name = 'exe%d' % (i)
    inner_exe_ext = ".exe"
    build_exe(name, inner_exe_ext, i, cc_flags, False, do_strip);
    if do_sign:
        cert_info = gen_cs_cert(name, inner_exe_ext)
        sign_exe(name, inner_exe_ext, cert_info)
    package_exe(name, inner_exe_ext)
    outer_exe_sources.append('build/%s%s.o' % (name, inner_exe_ext))

# Build the outer exe
name = 'test'
build_exe(name, ext, 0, cc_flags, do_res, do_strip, sources=outer_exe_sources)
if do_sign:
    cert_info = gen_cs_cert(name, ext)
    sign_exe(name, ext, cert_info)

# Delete unneeded artifacts
if do_cleanup:
    cleanups = ['build/*.o']
    if do_sign:
        cleanups += ['build/*.csr', 'build/*.key']

    if do_res:
        cleanups += ['build/test.exe.res']

    for cleanup in cleanups:
        cmd = 'rm %s' % (cleanup)
        try:
            run_cmd(cmd)
        except:
            pass



# A simple Python script to generate the code for odd-sized optimized DFTs
# The generated code is simply printed in the terminal.
# This is only intended for prime lengths, where the usual tricks can't be used.
# The generated code is O(n^2), but for short lengths this is still faster than fancier algorithms.
# Example, make a length 5 Dft:
# > python genbutterflies.py 5
# Output:
# let x14p = *buffer.get_unchecked(1) + *buffer.get_unchecked(4);
# let x14n = *buffer.get_unchecked(1) - *buffer.get_unchecked(4);
# let x23p = *buffer.get_unchecked(2) + *buffer.get_unchecked(3);
# let x23n = *buffer.get_unchecked(2) - *buffer.get_unchecked(3);
# let sum = *buffer.get_unchecked(0) + x14p + x23p;
# let b14re_a = buffer.get_unchecked(0).re + self.twiddle1.re*x14p.re + self.twiddle2.re*x23p.re;
# let b14re_b = self.twiddle1.im*x14n.im + self.twiddle2.im*x23n.im;
# let b23re_a = buffer.get_unchecked(0).re + self.twiddle2.re*x14p.re + self.twiddle1.re*x23p.re;
# let b23re_b = self.twiddle2.im*x14n.im + -self.twiddle1.im*x23n.im;
# 
# let b14im_a = buffer.get_unchecked(0).im + self.twiddle1.re*x14p.im + self.twiddle2.re*x23p.im;
# let b14im_b = self.twiddle1.im*x14n.re + self.twiddle2.im*x23n.re;
# let b23im_a = buffer.get_unchecked(0).im + self.twiddle2.re*x14p.im + self.twiddle1.re*x23p.im;
# let b23im_b = self.twiddle2.im*x14n.re + -self.twiddle1.im*x23n.re;
# 
# let out1re = b14re_a - b14re_b;
# let out1im = b14im_a + b14im_b;
# let out2re = b23re_a - b23re_b;
# let out2im = b23im_a + b23im_b;
# let out3re = b23re_a + b23re_b;
# let out3im = b23im_a - b23im_b;
# let out4re = b14re_a + b14re_b;
# let out4im = b14im_a - b14im_b;
# *buffer.get_unchecked_mut(0) = sum;
# *buffer.get_unchecked_mut(1) = Complex{ re: out1re, im: out1im };
# *buffer.get_unchecked_mut(2) = Complex{ re: out2re, im: out2im };
# *buffer.get_unchecked_mut(3) = Complex{ re: out3re, im: out3im };
# *buffer.get_unchecked_mut(4) = Complex{ re: out4re, im: out4im };
#
#
# This required the Butterfly5 to already exist, with twiddles defined like this:
# pub struct Butterfly5<T> {
#     twiddle1: Complex<T>,
#     twiddle2: Complex<T>,
# 	direction: FftDirection,
# }
# 
# With twiddle values:
# twiddle1: Complex<T> = twiddles::single_twiddle(1, 5, direction);
# twiddle2: Complex<T> = twiddles::single_twiddle(2, 5, direction);

import sys

def make_shuffling_single_f64(len):
    inputs =  ", ".join([str(n) for n in range(len)])
    print(f"let values = read_complex_to_array!(buffer, {{{inputs}}});")
    print("")
    print("let out = self.perform_fft_direct(values);")
    print("")
    print(f"write_complex_to_array!(out, buffer, {{{inputs}}});")

def make_shuffling_single_f32(len):
    inputs =  ", ".join([str(n) for n in range(len)])
    print(f"let values = read_partial1_complex_to_array!(buffer, {{{inputs}}});")
    print("")
    print("let out = self.perform_parallel_fft_direct(values);")
    print("")
    print(f"write_partial_lo_complex_to_array!(out, buffer, {{{inputs}}});")


def make_shuffling_parallel_f32(len):
    inputs =  ", ".join([str(2*n) for n in range(len)])
    outputs =  ", ".join([str(n) for n in range(len)])
    print(f"let input_packed = read_complex_to_array!(buffer, {{{inputs}}});")
    print("")
    print("let values = [")
    for n in range(int(len/2)):
        print(f"    extract_lo_hi_f32(input_packed[{int(n)}], input_packed[{int(len/2 + n)}]),")
        print(f"    extract_hi_lo_f32(input_packed[{int(n)}], input_packed[{int(len/2 + n+1)}]),")
    print(f"    extract_lo_hi_f32(input_packed[{int(len/2)}], input_packed[{int(len-1)}]),")
    print("];")
    print("")
    print("let out = self.perform_parallel_fft_direct(values);")
    print("")
    print("let out_packed = [")
    for n in range(int(len/2)):
        print(f"    extract_lo_lo_f32(out[{int(2*n)}], out[{int(2*n+1)}]),")
    print(f"    extract_lo_hi_f32(out[{int(len-1)}], out[0]),")
    for n in range(int(len/2)):
        print(f"    extract_hi_hi_f32(out[{int(2*n+1)}], out[{int(2*n+2)}]),")
    print("];")
    print("")
    print(f"write_complex_to_array_strided!(out_packed, buffer, 2, {{{outputs}}});")


def make_butterfly(len, fft2func, calcfunc, mulfunc, rotatefunc):


    halflen = int((fftlen+1)/2)

    for n in range(1, halflen):
        print(f"let [x{n}p{fftlen-n}, x{n}m{fftlen-n}] = {fft2func}(values[{n}], values[{fftlen-n}]);")



    print("")
    items = []
    for m in range (1, halflen):
        for n in range(1, halflen):
            mn = (m*n)%fftlen
            if mn > fftlen/2:
                mn = fftlen-mn
            print(f"let t_a{m}_{n} = {mulfunc}(self.twiddle{mn}re, x{n}p{fftlen-n});")

    print("")
    items = []
    for m in range (1, halflen):
        for n in range(1, halflen):
            mn = (m*n)%fftlen
            if mn > fftlen/2:
                mn = fftlen-mn
            print(f"let t_b{m}_{n} = {mulfunc}(self.twiddle{mn}im, x{n}m{fftlen-n});")
            

    print("")
    print("let x0 = values[0];")
    for m in range(1, halflen):
        items = ["x0"]
        for n in range(1, halflen):
            items.append(f"t_a{m}_{n}")
        terms = " + ".join(items)
        print(f'let t_a{m} = {calcfunc}({terms});')


    print("")
    for m in range(1, halflen):
        terms = f"t_b{m}_1"
        for n in range(2, halflen):
            mn = (m*n)%fftlen
            if mn > fftlen/2:
                sign = " - "
            else:
                sign = " + "
            terms = terms + sign + f"t_b{m}_{n}"
        print(f'let t_b{m} = {calcfunc}({terms});')

    print("")
    for m in range(1, halflen):
        print(f'let t_b{m}_rot = self.rotate.{rotatefunc}(t_b{m});')

    print("")
    items = ["x0"]
    for n in range(1, halflen):
        items.append(f"x{n}p{fftlen-n}")
    terms = " + ".join(items)
    print(f'let y0 = {calcfunc}({terms});')

    for m in range(1, halflen):
        print(f"let [y{m}, y{fftlen-m}] = {fft2func}(t_a{m}, t_b{m}_rot);")

    items = []
    for n in range(0, fftlen):
        items.append(f"y{n}")
    print(f'[{", ".join(items)}]')


if __name__ == "__main__":
    fftlen = int(sys.argv[1])
    print("\n\n--------------- f32 ---------------")
    print("\n ----- perform_fft_contiguous -----")
    make_shuffling_single_f32(fftlen)
    print("\n ----- perform_parallel_fft_contiguous -----")
    make_shuffling_parallel_f32(fftlen)
    print("\n ----- perform_parallel_fft_direct -----")
    make_butterfly(fftlen, "parallel_fft2_interleaved_f32", "calc_f32!", "_mm_mul_ps", "rotate_both")

    print("\n\n--------------- f64 ---------------")
    print("\n ----- perform_fft_contiguous -----")
    make_shuffling_single_f64(fftlen)
    print("\n ----- perform_parallel_fft_direct -----")
    make_butterfly(fftlen, "solo_fft2_f64", "calc_f64!", "_mm_mul_pd", "rotate")



#!/usr/bin/env python
# ===- lib/fuzzer/scripts/unbalanced_allocs.py ------------------------------===#
#
# Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
# See https://llvm.org/LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
#
# ===------------------------------------------------------------------------===#
#
# Post-process -trace_malloc=2 output and printout only allocations and frees
# unbalanced inside of fuzzer runs.
# Usage:
#   my_fuzzer -trace_malloc=2 -runs=10 2>&1 | unbalanced_allocs.py -skip=5
#
# ===------------------------------------------------------------------------===#

import argparse
import sys

_skip = 0


def PrintStack(line, stack):
    global _skip
    if _skip > 0:
        return
    print("Unbalanced " + line.rstrip())
    for l in stack:
        print(l.rstrip())


def ProcessStack(line, f):
    stack = []
    while line and line.startswith("    #"):
        stack += [line]
        line = f.readline()
    return line, stack


def ProcessFree(line, f, allocs):
    if not line.startswith("FREE["):
        return f.readline()

    addr = int(line.split()[1], 16)
    next_line, stack = ProcessStack(f.readline(), f)
    if addr in allocs:
        del allocs[addr]
    else:
        PrintStack(line, stack)
    return next_line


def ProcessMalloc(line, f, allocs):
    if not line.startswith("MALLOC["):
        return ProcessFree(line, f, allocs)

    addr = int(line.split()[1], 16)
    assert not addr in allocs

    next_line, stack = ProcessStack(f.readline(), f)
    allocs[addr] = (line, stack)
    return next_line


def ProcessRun(line, f):
    if not line.startswith("MallocFreeTracer: START"):
        return ProcessMalloc(line, f, {})

    allocs = {}
    print(line.rstrip())
    line = f.readline()
    while line:
        if line.startswith("MallocFreeTracer: STOP"):
            global _skip
            _skip = _skip - 1
            for _, (l, s) in allocs.items():
                PrintStack(l, s)
            print(line.rstrip())
            return f.readline()
        line = ProcessMalloc(line, f, allocs)
    return line


def ProcessFile(f):
    line = f.readline()
    while line:
        line = ProcessRun(line, f)


def main(argv):
    parser = argparse.ArgumentParser()
    parser.add_argument("--skip", default=0, help="number of runs to ignore")
    args = parser.parse_args()
    global _skip
    _skip = int(args.skip) + 1
    ProcessFile(sys.stdin)


if __name__ == "__main__":
    main(sys.argv)



# Copyright (C) 2020-2024 Cisco Systems, Inc. and/or its affiliates. All rights reserved.

"""
Run clamscan tests.
"""

import sys

sys.path.append('../unit_tests')
import testcase


class TC(testcase.TestCase):
    @classmethod
    def setUpClass(cls):
        super(TC, cls).setUpClass()

        TC.testpaths = list(TC.path_build.glob('unit_tests/input/clamav_hdb_scanfiles/clam*')) # A list of Path()'s of each of our generated test files

    @classmethod
    def tearDownClass(cls):
        super(TC, cls).tearDownClass()

    def setUp(self):
        super(TC, self).setUp()

    def tearDown(self):
        super(TC, self).tearDown()
        self.verify_valgrind_log()

    def test_yara(self):
        self.step_name('Test yara signature - detect TAR file magic in a range')

        db = TC.path_tmp / 'regex.yara'
        db.write_text(
            r'''
rule regex
{
    meta:
        author      = "Micah"
        date        = "2022/03/12"
        description = "Just a test"
    strings:
        $a = "/+eat/"                 /* <-- not a regex */
        $b = /\$protein+=\([a-z]+\)/  /* <-- is a regex */
    condition:
        all of them
}
            '''
        )
        testfile = TC.path_tmp / 'regex.sample'
        testfile.write_text('var $protein=(slugs); /+eat/ $protein')

        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan, path_db=db, testfiles=testfile,
        )
        output = self.execute_command(command)

        assert output.ec == 1  # virus found

        expected_results = [
            'regex.sample: YARA.regex.UNOFFICIAL FOUND',
            'Infected files: 1',
        ]
        self.verify_output(output.out, expected=expected_results)

    def test_slash_colon(self):
        self.step_name('Test LDB and Yara regex rules with / and : in the string work')
        # This is a regression test for a bug where :'s in a PCRE regex would act
        # as delimiters if there was also a / in the regex before the :

        testfile = TC.path_tmp / 'regex-slash-colon.sample'
        testfile.write_text('hello blee/blah: bleh')

        # First test with LDB PCRE rule
        #
        yara_db = TC.path_tmp / 'regex-slash-colon.ldb'
        yara_db.write_text(
            r'regex;Engine:81-255,Target:0;1;68656c6c6f20;0/hello blee\/blah: bleh/'
        )
        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan, path_db=yara_db, testfiles=testfile,
        )
        output = self.execute_command(command)

        assert output.ec == 1  # virus found

        expected_results = [
            'regex-slash-colon.sample: regex.UNOFFICIAL FOUND',
            'Infected files: 1',
        ]
        self.verify_output(output.out, expected=expected_results)

        # Second test with YARA regex rule
        #
        yara_db = TC.path_tmp / 'regex-slash-colon.yara'
        yara_db.write_text(
            r'''
rule regex
{
    meta:
        author      = "Micah"
        date        = "2022/07/25"
        description = "Just a test"
    strings:
        $b = /hello blee\/blah: bleh/
    condition:
        all of them
}
            '''
        )
        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan, path_db=yara_db, testfiles=testfile,
        )
        output = self.execute_command(command)

        assert output.ec == 1  # virus found

        expected_results = [
            'regex-slash-colon.sample: YARA.regex.UNOFFICIAL FOUND',
            'Infected files: 1',
        ]
        self.verify_output(output.out, expected=expected_results)

    def test_ldb_offset_pcre(self):
        self.step_name('Test LDB regex rules with an offset')
        # The offset feature starts the match some # of bytes after start of the pattern match
        # The offset is EXACT, meaning it's no longer wildcard.
        # The match must occur exactly that number of bytes from the start of the file.

        # using 'MZ' prefix so it is detected as MSEXE and not TEXT. This is to avoid normalization.
        testfile = TC.path_tmp / 'ldb_offset_pcre'
        testfile.write_text('MZ hello blee')

        # First without the offset, make sure it matches
        yara_db = TC.path_tmp / 'ldb_pcre_no_offset.ldb'
        yara_db.write_text(
            r'ldb_pcre_no_offset;Engine:81-255,Target:0;0&1;68656c6c6f20;0/hello blee/'
        )
        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan, path_db=yara_db, testfiles=testfile,
        )
        output = self.execute_command(command)

        assert output.ec == 1  # virus found

        expected_results = [
            'ldb_offset_pcre: ldb_pcre_no_offset.UNOFFICIAL FOUND',
            'Infected files: 1',
        ]

        # Next, with the offset, but it won't match, because the regex pattern is "hello blee"
        # and with the offset of 5 (from start of file) means it should start the pcre matching at "llo blee"
        yara_db = TC.path_tmp / 'ldb_pcre_offset_no_match.ldb'
        yara_db.write_text(
            r'ldb_pcre_offset_no_match;Engine:81-255,Target:0;0&1;68656c6c6f20;5:0/hello blee/'
        )
        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan, path_db=yara_db, testfiles=testfile,
        )
        output = self.execute_command(command)

        assert output.ec == 0  # virus NOT found

        expected_results = [
            'ldb_offset_pcre: OK',
        ]

        # Next, with the offset, and it SHOULD match, because the regex pattern is "llo blee"
        # and with the offset of 5 (from start of file) means it should start the pcre matching at "llo blee"
        yara_db = TC.path_tmp / 'ldb_pcre_offset_match.ldb'
        yara_db.write_text(
            r'ldb_pcre_offset_match;Engine:81-255,Target:0;0&1;68656c6c6f20;5:0/llo blee/'
        )
        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan, path_db=yara_db, testfiles=testfile,
        )
        output = self.execute_command(command)

        assert output.ec == 1  # virus found

        expected_results = [
            'ldb_offset_pcre: ldb_pcre_offset_match.UNOFFICIAL FOUND',
            'Infected files: 1',
        ]

    def test_pcre_flag(self):
        self.step_name('Test LDB regex rules with case insensitive flag')
        # This test validates that the flags field is, and more specifically the case-insensitive flag is working.

        # using 'MZ' prefix so it is detected as MSEXE and not TEXT. This is to avoid normalization.
        testfile = TC.path_tmp / 'ldb_pcre_flag'
        testfile.write_text('MZ hello blee / BlAh')

        # First test withOUT the case-insensitive flag. It should NOT match.
        yara_db = TC.path_tmp / 'ldb_pcre_case.ldb'
        yara_db.write_text(
            r'ldb_pcre_case;Engine:81-255,Target:0;0&1;68656c6c6f20;0/blah/'
        )
        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan, path_db=yara_db, testfiles=testfile,
        )
        output = self.execute_command(command)

        assert output.ec == 0  # virus NOT found

        expected_results = [
            'ldb_pcre_flag: OK',
        ]

        # First test WITH the case-insensitive flag. It SHOULD match.
        yara_db = TC.path_tmp / 'ldb_pcre_nocase.ldb'
        yara_db.write_text(
            r'ldb_pcre_nocase;Engine:81-255,Target:0;0&1;68656c6c6f20;0/blah/i'
        )
        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan, path_db=yara_db, testfiles=testfile,
        )
        output = self.execute_command(command)

        assert output.ec == 1  # virus found

        expected_results = [
            'ldb_pcre_flag: ldb_pcre_nocase.UNOFFICIAL FOUND',
            'Infected files: 1',
        ]

    def test_ldb_multi_pcre(self):
        self.step_name('Test LDB and Yara regex rules with / and : in the string work')
        # This is a regression test for a bug where :'s in a PCRE regex would act
        # as delimiters if there was also a / in the regex before the :

        # using 'MZ' prefix so it is detected as MSEXE and not TEXT. This is to avoid normalization.
        testfile = TC.path_tmp / 'ldb_multi_pcre'
        testfile.write_text('MZ hello blee / BlAh')

        # Verify first with two subsigs that should match, that the alert has found.
        yara_db = TC.path_tmp / 'ldb_multi_pcre.ldb'
        yara_db.write_text(
            r'ldb_multi_pcre;Engine:81-255,Target:0;0&1&2;68656c6c6f20;0/hello blee/;0/blah/i'
        )
        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan, path_db=yara_db, testfiles=testfile,
        )
        output = self.execute_command(command)

        assert output.ec == 1  # virus found

        expected_results = [
            'ldb_multi_pcre: ldb_multi_pcre.UNOFFICIAL FOUND',
            'Infected files: 1',
        ]

        # Verify next that if one of the two subsigs do not match, the whole thing does not match.
        yara_db = TC.path_tmp / 'ldb_multi_pcre.ldb'
        yara_db.write_text(
            r'ldb_multi_pcre;Engine:81-255,Target:0;0&1&2;68656c6c6f20;0/hello blee/;0/bloh/i'
        )
        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan, path_db=yara_db, testfiles=testfile,
        )
        output = self.execute_command(command)

        assert output.ec == 0  # virus NOT found

        expected_results = [
            'ldb_multi_pcre: OK',
            'Infected files: 0',
        ]

        # Verify next that if the other of the two subsigs do not match, the whole thing does not match.
        yara_db = TC.path_tmp / 'ldb_multi_pcre.ldb'
        yara_db.write_text(
            r'ldb_multi_pcre;Engine:81-255,Target:0;0&1&2;68656c6c6f20;0/hella blee/;0/blah/i'
        )
        command = '{valgrind} {valgrind_args} {clamscan} -d {path_db} {testfiles}'.format(
            valgrind=TC.valgrind, valgrind_args=TC.valgrind_args, clamscan=TC.clamscan, path_db=yara_db, testfiles=testfile,
        )
        output = self.execute_command(command)

        assert output.ec == 0  # virus NOT found

        expected_results = [
            'ldb_multi_pcre: OK',
            'Infected files: 0',
        ]



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  cdef struct Opaque:
    pass

  cdef struct Foo_u64:
    float *a;
    uint64_t *b;
    Opaque *c;
    uint64_t **d;
    float **e;
    Opaque **f;
    uint64_t *g;
    int32_t *h;
    int32_t **i;

  void root(int32_t *arg, Foo_u64 *foo, Opaque **d);



#if 0
DEF FOO = 0
DEF BAR = 0
#endif


from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  IF FOO:
    const int32_t FOO # = 1

  IF BAR:
    const int32_t BAR # = 2

  IF FOO:
    cdef struct Foo:
      pass

  IF BAR:
    cdef struct Bar:
      pass

  IF FOO:
    void foo(const Foo *foo);

  IF BAR:
    void bar(const Bar *bar);



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  cdef enum:
    a,
    b,
  ctypedef uint8_t Enum;

  ctypedef struct Struct:
    Enum field;

  extern const Enum STATIC;

  void fn(Struct arg);



import sys
import math
from matplotlib import pyplot as plt


with open(sys.argv[1]) as f:
    lines = f.readlines()

results = {"f32": {"scalar": {}, "wasmsimd": {}}, "f64": {"scalar": {}, "wasmsimd": {}}}

for line in lines:
    if line.startswith("test ") and not line.startswith("test result"):
        name, result = line.split("... bench:")
        name = name.split()[1]
        _, length, ftype, algo = name.split("_")
        value = float(result.strip().split(" ")[0].replace(",", ""))
        results[ftype][algo][float(length)] = value

lengths = sorted(list(results["f32"]["scalar"].keys()))


scalar_32 = []
wasmsimd_32 = []
for l in lengths:
    sc32 = results["f32"]["scalar"][l]
    nn32 = results["f32"]["wasmsimd"][l]
    scalar_32.append(100.0)
    wasmsimd_32.append(100.0 * sc32/nn32)

scalar_64 = []
wasmsimd_64 = []
for l in lengths:
    sc64 = results["f64"]["scalar"][l]
    nn64 = results["f64"]["wasmsimd"][l]
    scalar_64.append(100.0)
    wasmsimd_64.append(100.0 * sc64/nn64)

lengths = [math.log(l, 2) for l in lengths]

plt.figure()
plt.plot(lengths, scalar_64, lengths, wasmsimd_64)
plt.title("f64")
plt.ylabel("relative speed, %")
plt.xlabel("log2(length)")
plt.xticks(list(range(4,23)))
plt.grid()
plt.legend(["scalar", "wasmsimd"])

plt.figure()
plt.plot(lengths, scalar_32, lengths, wasmsimd_32)
plt.title("f32")
plt.ylabel("relative speed, %")
plt.xlabel("log2(length)")
plt.legend(["scalar", "wasmsimd"])
plt.xticks(list(range(4,23)))
plt.grid()
plt.show()





from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list

cdef extern from *:

  cdef enum:
    IV,
  ctypedef ptrdiff_t IE;

  cdef enum:
    UV,
  ctypedef size_t UE;

  ctypedef size_t Usize;

  ctypedef ptrdiff_t Isize;

  void root(Usize, Isize, UE, IE);



from libc.stdint cimport int8_t, int16_t, int32_t, int64_t, intptr_t
from libc.stdint cimport uint8_t, uint16_t, uint32_t, uint64_t, uintptr_t
cdef extern from *:
  ctypedef bint bool
  ctypedef struct va_list
#ifndef NO_RETURN_ATTR
  #ifdef __GNUC__
    #define NO_RETURN_ATTR __attribute__ ((noreturn))
  #else // __GNUC__
    #define NO_RETURN_ATTR
  #endif // __GNUC__
#endif // NO_RETURN_ATTR


cdef extern from *:

  ctypedef struct Example:
    void (*f)(uintptr_t, uintptr_t);

  void loop_forever();

  uint8_t normal_return(Example arg, void (*other)(uint8_t));
